@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  * 02111-1307 USA
  *
- * $Id: network_stats.c,v 1.72 2005/02/24 18:35:11 tdb Exp $
+ * $Id: network_stats.c,v 1.75 2005/07/30 13:23:46 ats Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -49,9 +49,10 @@
 #include <sys/socket.h>
 #include <net/if.h>
 #include <ctype.h>
-/* Stuff which could be defined by defining KERNEL, but 
- * that would be a bad idea, so we'll just declare it here
- */
+#include <linux/version.h>
+#include <asm/types.h>
+/* These aren't defined by asm/types.h unless the kernel is being
+   compiled, but some versions of ethtool.h need them. */
 typedef __uint8_t u8;
 typedef __uint16_t u16;
 typedef __uint32_t u32;
@@ -406,7 +407,7 @@ int sg_network_io_compare_name(const void *va, const void *vb) {
 static void network_iface_stat_init(sg_network_iface_stats *s) {
 	s->interface_name = NULL;
 	s->speed = 0;
-	s->dup = SG_IFACE_DUPLEX_UNKNOWN;
+	s->duplex = SG_IFACE_DUPLEX_UNKNOWN;
 }
 
 static void network_iface_stat_destroy(sg_network_iface_stats *s) {
@@ -473,7 +474,7 @@ sg_network_iface_stats *sg_get_network_iface_stats(int *entries){
 		}
 
 		network_iface_stat_ptr->speed = 0;
-		network_iface_stat_ptr->dup = SG_IFACE_DUPLEX_UNKNOWN;
+		network_iface_stat_ptr->duplex = SG_IFACE_DUPLEX_UNKNOWN;
 		ifaces++;
 
 		memset(&ifmed, 0, sizeof(struct ifmediareq));
@@ -530,11 +531,11 @@ sg_network_iface_stats *sg_get_network_iface_stats(int *entries){
 		}
 
 		if( (ifmed.ifm_active | IFM_FDX) == ifmed.ifm_active ){
-			network_iface_stat_ptr->dup = SG_IFACE_DUPLEX_FULL;
+			network_iface_stat_ptr->duplex = SG_IFACE_DUPLEX_FULL;
 		}else if( (ifmed.ifm_active | IFM_HDX) == ifmed.ifm_active ){
-			network_iface_stat_ptr->dup = SG_IFACE_DUPLEX_HALF;
+			network_iface_stat_ptr->duplex = SG_IFACE_DUPLEX_HALF;
 		}else{
-			network_iface_stat_ptr->dup = SG_IFACE_DUPLEX_UNKNOWN;
+			network_iface_stat_ptr->duplex = SG_IFACE_DUPLEX_UNKNOWN;
 		}
 
 	}	
@@ -588,14 +589,14 @@ sg_network_iface_stats *sg_get_network_iface_stats(int *entries){
 				network_iface_stat_ptr->speed = 0;
 			}
 
-			network_iface_stat_ptr->dup = SG_IFACE_DUPLEX_UNKNOWN;
+			network_iface_stat_ptr->duplex = SG_IFACE_DUPLEX_UNKNOWN;
 			if ((knp = kstat_data_lookup(ksp, "link_duplex")) != NULL) {
 				switch (knp->value.ui32) {
 				case 1:
-					network_iface_stat_ptr->dup = SG_IFACE_DUPLEX_HALF;
+					network_iface_stat_ptr->duplex = SG_IFACE_DUPLEX_HALF;
 					break;
 				case 2:
-					network_iface_stat_ptr->dup = SG_IFACE_DUPLEX_FULL;
+					network_iface_stat_ptr->duplex = SG_IFACE_DUPLEX_FULL;
 					break;
 				}
 			}
@@ -676,18 +677,18 @@ sg_network_iface_stats *sg_get_network_iface_stats(int *entries){
 
 			switch (ethcmd.duplex) {
 			case DUPLEX_FULL:
-				network_iface_stat_ptr->dup = SG_IFACE_DUPLEX_FULL;
+				network_iface_stat_ptr->duplex = SG_IFACE_DUPLEX_FULL;
 				break;
 			case DUPLEX_HALF:
-				network_iface_stat_ptr->dup = SG_IFACE_DUPLEX_HALF;
+				network_iface_stat_ptr->duplex = SG_IFACE_DUPLEX_HALF;
 				break;
 			default:
-				network_iface_stat_ptr->dup = SG_IFACE_DUPLEX_UNKNOWN;
+				network_iface_stat_ptr->duplex = SG_IFACE_DUPLEX_UNKNOWN;
 			}
 		} else {
 			/* Not all interfaces support the ethtool ioctl. */
 			network_iface_stat_ptr->speed = 0;
-			network_iface_stat_ptr->dup = SG_IFACE_DUPLEX_UNKNOWN;
+			network_iface_stat_ptr->duplex = SG_IFACE_DUPLEX_UNKNOWN;
 		}
 
 		ifaces++;
@@ -704,6 +705,10 @@ sg_network_iface_stats *sg_get_network_iface_stats(int *entries){
 	return NULL;
 #endif
 
+#ifdef SG_ENABLE_DEPRECATED
+	network_iface_stat_ptr->dup = network_iface_stat_ptr->duplex;
+#endif
+
 	*entries = ifaces;
 	return network_iface_stats; 
 }
