@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  * 02111-1307 USA
  *
- * $Id: page_stats.c,v 1.21 2004/04/07 21:08:40 tdb Exp $
+ * $Id: page_stats.c,v 1.22 2004/07/18 21:30:11 ats Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -119,19 +119,21 @@ sg_page_stats *sg_get_page_stats(){
 
 		fclose(f);
 	} else {
-		sg_set_error(SG_ERROR_OPEN, "/proc/stat");
+		sg_set_error_with_errno(SG_ERROR_OPEN, "/proc/stat");
 		return NULL;
 	}
 #endif
 #if defined(FREEBSD) || defined(DFBSD)
 	size = sizeof page_stats.pages_pagein;
 	if (sysctlbyname("vm.stats.vm.v_swappgsin", &page_stats.pages_pagein, &size, NULL, 0) < 0){
-		sg_set_error(SG_ERROR_SYSCTLBYNAME, "vm.stats.vm.v_swappgsin");
+		sg_set_error_with_errno(SG_ERROR_SYSCTLBYNAME,
+		                        "vm.stats.vm.v_swappgsin");
 		return NULL;
 	}
 	size = sizeof page_stats.pages_pageout;
 	if (sysctlbyname("vm.stats.vm.v_swappgsout", &page_stats.pages_pageout, &size, NULL, 0) < 0){
-		sg_set_error(SG_ERROR_SYSCTLBYNAME, "vm.stats.vm.v_swappgsout");
+		sg_set_error_with_errno(SG_ERROR_SYSCTLBYNAME,
+		                        "vm.stats.vm.v_swappgsout");
 		return NULL;
 	}
 #endif
