@@ -18,7 +18,7 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
  * 02111-1307 USA
  *
- * $Id: tools.h,v 1.24 2004/04/08 10:56:13 tdb Exp $
+ * $Id: tools.h,v 1.25 2004/11/06 14:55:53 ats Exp $
  */
 
 #include <stdio.h>
@@ -31,6 +31,10 @@
 #ifdef NETBSD
 #include <uvm/uvm_extern.h>
 #endif
+#ifdef HPUX
+#include <sys/param.h>
+#include <sys/pstat.h>
+#endif
 
 #ifdef SOLARIS
 const char *sg_get_svr_from_bsd(const char *bsd);
@@ -57,6 +61,10 @@ kvm_t *sg_get_kvm2(void);
 struct uvmexp *sg_get_uvmexp(void);
 #endif
 
+#ifdef HPUX
+struct pst_static *sg_get_pstat_static(void);
+#endif
+
 void *sg_realloc(void *ptr, size_t size);
 #define sg_malloc(size) sg_realloc(NULL, size)
 
