@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * $Id: statgrab.c,v 1.32 2004/08/10 21:10:55 ats Exp $
+ * $Id: statgrab.c,v 1.35 2005/07/13 13:01:24 tdb Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -313,6 +313,20 @@ void populate_fs() {
 				 "fs", name, "used_inodes", NULL);
 			add_stat(LONG_LONG, &disk[i].free_inodes,
 				 "fs", name, "free_inodes", NULL);
+			add_stat(LONG_LONG, &disk[i].avail_inodes,
+				 "fs", name, "avail_inodes", NULL);
+			add_stat(LONG_LONG, &disk[i].io_size,
+				 "fs", name, "io_size", NULL);
+			add_stat(LONG_LONG, &disk[i].block_size,
+				 "fs", name, "block_size", NULL);
+			add_stat(LONG_LONG, &disk[i].total_blocks,
+				 "fs", name, "total_blocks", NULL);
+			add_stat(LONG_LONG, &disk[i].free_blocks,
+				 "fs", name, "free_blocks", NULL);
+			add_stat(LONG_LONG, &disk[i].avail_blocks,
+				 "fs", name, "avail_blocks", NULL);
+			add_stat(LONG_LONG, &disk[i].used_blocks,
+				 "fs", name, "used_blocks", NULL);
 
 			free(buf);
 		}
@@ -412,7 +426,7 @@ void populate_net() {
 				 "net", name, "speed", NULL);
 			add_stat(BOOL, &iface[i].up,
 				 "net", name, "up", NULL);
-			add_stat(DUPLEX, &iface[i].dup,
+			add_stat(DUPLEX, &iface[i].duplex,
 				 "net", name, "duplex", NULL);
 		}
 	}
@@ -639,7 +653,7 @@ void usage() {
 	       "  -t DELAY   When repeating, wait DELAY seconds between updates (default 1)\n"
 	       "  -p         Display CPU usage differences as percentages rather than\n"
 	       "             absolute values\n"
-	       "  -f SCALE   Display floating-point values as integers scaled by FACTOR\n"
+	       "  -f FACTOR  Display floating-point values as integers scaled by FACTOR\n"
 	       "  -K         Display byte counts in kibibytes\n"
 	       "  -M         Display byte counts in mebibytes\n"
 	       "  -G         Display byte counts in gibibytes\n"
